#######################################################
#Hot Gradient
#######################################################
palette.Hot = function(n, start = 0, end = 1, gamma = 1.1){
   x = seq(start,end,length.out=n);
   y = matrix(0,length(x),3);
   
   l=which(x<=3/8);
   y[l,1]=x[l]*8/3;

   l=which(x>=3/8 & x<=6/8);
   y[l,1]=1;
   y[l,2]=(x[l]-3/8)*8/3;
   y[l,3]=0;

   l=which(x>=6/8);
   y[l,1]=1;
   y[l,2]=1;
   y[l,3]=(x[l]-6/8)*8/6;

   return(y)
}
colors.Hot= function(...){ rgb(palette.Hot(...))}
color.Hot= function(i,m){ rgb(palette.Hot(1,i/m,i/m))}
#barplot(rep(100,100),col=colors.Hot(100),border=NA,space=0,axes=F)



#######################################################
#Soft Gradient
#######################################################
palette.SoftGrad = function(n, start = 0, end = 1, gamma = 1.1){
   x=seq(start*100,end*100,length.out=n);

   l=0.439216;
   h=0.666667;
   xC=c(0,18,45,72,100)
   yR=c(l,l,l,h,h);
   yG=c((l+h)/2,h,h,h,l);
   yB=c(h,h,l,l,l);
   return(t(rbind(approx(xC,yR,x)$y,   approx(xC,yG,x)$y,   approx(xC,yB,x)$y)))
}
colors.SoftGrad= function(...){ rgb(palette.SoftGrad(...))}
color.SoftGrad= function(i,m){ rgb(palette.SoftGrad(1,i/m,i/m))}
colored.SoftGrad=function(x){
  M=max(x)
  m=min(x)
  mu=mean(x)
  n=length(x)
  y=NULL;
  for(i in 1:n){
	y=c(y,colors.SoftGrad(1,(x[i]-m)/(M-m) ,(x[i]-m)/(M-m)) )
  }
  return(y);
}
#barplot(rep(100,100),col=colors.SoftGrad(100),border=NA,space=0,axes=F)


#######################################################
#Pastel Gradient
#######################################################
palette.Pastel= function(n, start = 0, end = 1, gamma = 1.1){
   x=seq(start*100,end*100,length.out=n);
  return(Keep01(t(rbind( 0.7674 + 0.00196665*x + 0.000504382*x^2 - 0.0000251222*x^3 + 5.33162*10^-7*x^4 - 5.31167*10^-9*x^5 +    1.9351*10^-11*x^6,
  0.461581 + 0.0195207*x - 0.00089656*x^2 + 0.0000264268*x^3 - 3.35541*10^-7*x^4 + 1.61372*10^-9*x^5 - 1.74239*10^-12*x^6  ,
  0.930945 + 0.0173366*x - 0.00276449*x^2 + 0.000102821*x^3 - 1.7649*10^-6*x^4 + 1.4871*10^-8*x^5 - 4.91633*10^-11*x^6 ))))
}
colors.Pastel= function(...){ rgb(palette.Pastel(...))}
color.Pastel= function(i,m){ rgb(palette.Pastel(1,i/m,i/m))}
#barplot(rep(100,100),col=colors.Pastel(100),border=NA,space=0,axes=F)






#######################################################
#Light Temperature Map Gradient
#######################################################
palette.TempLight= function(n, start = 0, end = 1, gamma = 1.1){
   x=seq(start*100,end*100,length.out=n);
  return(Keep01(t(rbind( 0.150213 + 0.0251463 *x - 0.000885904 *x^2 + 0.0000376188 *x^3 -     7.35876*10^-7 *x^4 + 6.17831*10^-9 *x^5 - 1.87701*10^-11 *x^6,
 0.280603 + 0.0278212 *x - 0.000120012 *x^2 - 8.66769*10^-6 *x^3 +   1.86101*10^-7 *x^4 - 1.74555*10^-9 *x^5 + 6.01835*10^-12 *x^6,
 0.940089 + 0.00149884 *x - 0.000265195 *x^2 + 0.0000232296 *x^3 -  6.59952*10^-7 *x^4 + 6.93436*10^-9 *x^5 - 2.484*10^-11 *x^6 ))))
}
colors.TempLight= function(...){ rgb(palette.TempLight(...))}
color.TempLight= function(i,m){ rgb(palette.TempLight(1,i/m,i/m))}
#barplot(rep(100,100),col=colors.TempLight(100),border=NA,space=0,axes=F)





#######################################################
# Indexed
#######################################################
palette.Indexed= function(n, start = 0, end = 1, gamma = 1.1){
   x=seq(start*100,end*100,length.out=n);

   cols=c(0.803922, 0.309804, 0.0705882, 1., 0.72549, 0.0941176, 0.956863, 0.796078, 0.137255, 0.952941, 0.980392, 0.572549, 0.411765, 0.592157, 0.4,
       0.309804, 0.470588, 0.345098, 0.298039, 0.286275, 0.54902, 0.270588, 0.184314, 0.376471, 0.878431, 0.101961, 0.207843, 0.8, 0.0588235, 0.0745098)
   Y=matrix(cols,10,3,byrow=T)
   yR=Y[,1];
   yG=Y[,2];
   yB=Y[,3];
   xC=seq(0,100,length.out=10);

    return(t(rbind(approx(xC,yR,x)$y,   approx(xC,yG,x)$y,   approx(xC,yB,x)$y)))

}
colors.Indexed= function(...){ rgb(palette.Indexed(...))}
color.Indexed= function(i,m){ rgb(palette.Indexed(1,i/m,i/m))}
#barplot(rep(100,100),col=colors.Indexed(100),border=NA,space=0,axes=F)



#######################################################
# Indexed0
#######################################################
palette.Indexed0= function(n, start = 0, end = 1, gamma = 1.1){
   x=seq(start*100,end*100,length.out=n);

   cols=
   c(0.2472, 0.24, 0.6,
   0.6, 0.24, 0.442893,
   0.6, 0.547014, 0.24,
   0.24, 0.6, 0.33692)
   Y=matrix(cols,4,3,byrow=T)
   yR=Y[,1];
   yG=Y[,2];
   yB=Y[,3];
   xC=seq(0,100,length.out=4);

    return(t(rbind(approx(xC,yR,x)$y,   approx(xC,yG,x)$y,   approx(xC,yB,x)$y)))

}
colors.Indexed0= function(...){ rgb(palette.Indexed0(...))}
color.Indexed0= function(i,m){ rgb(palette.Indexed0(1,i/m,i/m))}
#barplot(rep(100,100),col=colors.Indexed0(100),border=NA,space=0,axes=F)



Keep01=function(x){
  x[which(x>1)]=1;
  x[which(x<0)]=0;
  return(x)
}