
###################################################################
# ALL THE SPECTRUM FUNCTIONS RESTURN THE 0 FREQUENCY
#
###################################################################


####################################################################
# The TWO following methods return the exact same things
# No 2 pi factor are present to normalize the spectrum
# The integral of S with respect to f returns the turbulent kinetic energy
# kt =0.5 mean(u'^2)
# THIS IS ONLY TRUE IF THE MEAN IS ZERO
###################################################################
getPowerSpectrum=function(y,fs){
	R=list()
	N=length(y)
	Ncut=floor(N/2+1);
	R$f=(0:floor(N/2))*fs/N;
	R$S=1/(N*fs)*(Mod(fft(y))^2)[1:Ncut]
	R$S[1]=R$S[1]/4
	R$S[Ncut]=R$S[Ncut]/4

        spectrumCheckPrint(R,fs,N)
	invisible(R)
}
#Using Amplitude definitions
getPowerSpectrumWithAmplitudes=function(y,fs){
	R=list()
	N=length(y)
	Ncut=floor(N/2+1);
	R$f=(0:floor(N/2))*fs/N;
	Y=fft(y) /N
	R$A=(Mod(c( Y[1], 2*Y[2:floor(N/2)] ,Y[Ncut]  )));
	R$S=N/(4*fs) * R$A^2  # almost fft^2/(N*fs);
        spectrumCheckPrint(R,fs,N)
	invisible(R)
}

#
getFourrierCoefficients=function(y,fs){
	R=list()
	N=length(y)
	Ncut=floor(N/2+1);
	R$f=(0:floor(N/2))*fs/N;
        R$ff=(floor(N/2):floor(N/2))*fs/N;
        R$c=fft(y)/N
        R$a=Re(R$c)
        R$b=Im(R$c)
	invisible(R)
}




####################################################################
#
###################################################################
### From ECN
SingleSidedPowerDensitySpectrum=function(y,fs){
  R=list()
  N=length(y)
  R$f=(0:floor(N/2))*fs/N;
  DSP = abs(fft(y))^2
  DSP= 2*DSP/(N*fs)[1:length(f)]  #L*Fs is KMU and 2 is for compensation of the negative frequencies
  #removing the 1st value
  R$S=DSP[1:length(R$f)]
  spectrumCheckPrint(R,fs,N)
  invisible(R)
}
#SingleSidedPowerDensitySpectrum(up1,fs)

### TO BE DONE
SpectrumSingleSided=function(y,fs){
  #### to be done
  # y Signal, fs Sampling frequency
  invisible(R)
}

### From ECN
SingleSidedAmplitudeSpectrum=function(y,fs){
  N = length(y); # Length of signal
  f = fs/2*(seq(0,1,length.out=(N/2)));
  Y =(2*abs(fft(y))/N) [1:length(f)]     #times 2 for compensation of the negative frequencies
  #removing the 1st value
  f=f[2:length(f)]
  Y=Y[2:length(Y)]
  invisible(cbind(f,Y))
}


####################################################################
#  The two following return the same, exept maybe for the first value
###################################################################
### Compatible with Jakob Mann Spectrum function
SpectrumDoubleSided=function(y,fs,N=round(length(y)/1)){
  # y Signal, fs Sampling frequency, N length of sub samples
  R=list();
  # Cyclic frequencies by step of fs/2.
  R$f = fs/2*(seq(0,1,length.out=(N/2+1)));
  # Number of sub-samples (most of the time 1)
  K=floor(length(y)/N);
  Samples=matrix(y[1:(N*K)],K,N,byrow=T)
  S=matrix(NA,K,N/2+1)
  for(i in 1:K){
          # Double sided spectrum
          S[i,]= ( abs(fft(Samples[i,]))^2/(N*fs) )[1:(N/2+1)]
  }
  #averaging sub-samples
  R$S=colMeans(S);

  # Printing for checking
  spectrumCheckPrint(R,fs,N)
  # return
  invisible(R)
}


getPowerSpectrumManual=function(y,fs){
	R=list()
	N=length(y)
	Ncut=floor(N/2+1);
	R$f=(0:floor(N/2))*fs/N;
	I=complex(real=0,imaginary=1)
	S=numeric(Ncut)
	pb=txtProgressBar(0,Ncut,style=3)
	for(l in 0:(Ncut-1)){
		S[l+1]=1/(N*fs)*Mod( sum( y[1:N]*exp(-I*2*pi*l*(0:(N-1))/N) ) )^2
		setTxtProgressBar(pb,l)
	}
	close(pb)
	R$S=S;
        spectrumCheckPrint(R,fs,N)
	invisible(R)
}






####################################################################
# FOURIER HARMONICS
###################################################################
getFourrierCoefficients=function(y,fs){
	R=list()
	N=length(y)
	R$f=(0:floor(N/2))*fs/N;
        R$c=(fft(y)/N)[1:(N/2+1)]
        R$a=2*Re(R$c)
        R$a[1]=R$a[1]/2;
        R$b=-2*Im(R$c)
	invisible(R)
}


ReconstructSignalFromHarmonics=function(t, a, b, fs, ord=(length(a)-1)){
   N=length(t);
   ord=ord+1;
   k <- (2 * pi * t/N * fs) %*% t(0:(ord-1))
   y <-   cos(k) %*% a[1:ord] +
          sin(k) %*% b[1:ord]
   y
}
ReconstructSignalFromFFT=function(Yfft,N){
  #Perform the inverse fourier transform, assuming Yfft=fft(y)
  if(length(Yfft!=N)){ Yfft=c(Yfft,numeric(N-length(Yfft))) }
  2*Re(fft(Yfft, inverse=TRUE)/N) - Re(Yfft[1])/N
}

##################################################################
# Correlation
##################################################################
timeCorrelation=function(x,fs=1){
  j=0
  R=list()
  R$tau=numeric()
  R$R=numeric()
  rho=1
  while(rho>0){
     j=j+1;
     rho=cor(x[1:(length(x)-j+1)],x[j:length(x)])
     R$R[j]=rho
  }
  R$tau=(1/fs)*0:(length(R$R)-1)
  return(R)
}
#R=timeCorrelation(up1,fs)

##################################################################
#  Auxiliary functions
##################################################################
Drop.f0=function(R){
	if(R$f[1]==0){
	    for(i in 1:length(R)){
              if(length(R[[i]])>2){  R[[i]]=R[[i]][-1]}
             }
        }
        invisible(R)
}

LogSmooth=function(R,PointsPerDecades=15){
	if(min(R$f)==0){ stop("Drop the zero frequency !") }
        decades=range.decade(R$f)
	Ndecades=decades[2]-decades[1];
	M=data2bins(log(R$f),log(R$S),N=Ndecades*PointsPerDecades,BeNiceWithSingleton=T)
	R$fb   = rm.na(exp(M[,1]))
	R$Sb   = rm.na(exp(M[,2]))
	invisible(R);
}


spectrumCheckPrint=function(R,fs,N){
        # Printing for checking
        temp=cbind(R$f[1:6],R$S[1:6])
        colnames(temp)<-c("f","S")
        print(temp)
        temp=cbind(R$f[(N/2-4):(N/2+1)],R$S[(N/2-4):(N/2+1)])
        colnames(temp)<-c("f","S")
        print(temp)
	print(sum(R$S)*fs/N)
	print(trapz(R$f,R$S))
}


