

#################################################################################
# multiplot
#################################################################################
multiBinsPlot=function(M,xlim=NULL,ylim=NULL,xlab="",ylab="",main="",pch="",lty="",
                      legendNames=NULL,legendPos=NULL,Vcol=4,Vpch=1,Vlty=1,XY=F,...){
  #defining limits from X and the first columns of Y
  n=dim(M)[1]
  l=dim(M)[2]
  if(is.null(ylim)){
    mini=NULL;
    maxi=NULL;
    for(j in 1:(floor(l/3))){
      auxmin=min(M[,(j-1)*3+2],na.rm=T)
      auxmax=max(M[,(j-1)*3+2],na.rm=T)
      if(is.null(mini) || auxmin<mini){mini=auxmin}
      if(is.null(maxi) || auxmax>maxi){maxi=auxmax}
    }
    ylim=c(mini,maxi)
  }

  plot(M[,1],M[,2],xlim=xlim,ylim=ylim,axes=FALSE,xlab=xlab,ylab=ylab,main=main,pch=Vpch[1],col=Vcol[1],type="o")
  for(j in (2:floor(l/3))){
    par(new=TRUE)
    plotWithGrid(M[,(j-1)*3+1],M[,(j-1)*3+2],xlim[1],xlim[2],ylim[1],ylim[2],Xgrid=20,Ygrid=20,xlab="",ylab="",main="",pch=Vpch[(j-1)%%length(Vpch)+1],lty=Vlty[(j-1)%%(length(Vlty))+1],col=Vcol[(j-1)%%(length(Vlty))+1],type="o")

  }
  if(is.null(legendNames)){
    legendNames=1:floor(n/3)
  }
  if(is.null(legendPos)){
    legendPos="topright";
  }
  if(XY){
    par(new=TRUE)
    X=seq(xlim[1],xlim[2],(xlim[2]-xlim[1])/100)
    plotWithGrid(X,X,xlim[1],xlim[2],ylim[1],ylim[2],Xgrid=20,Ygrid=20,xlab="",ylab="",main="",pch="",lty=1,col=1,type="l")
    legendNames=c(legendNames,"Y=X")
    Vcol=c(Vcol,1)
    Vpch=c(Vpch,-1)
    Vlty=c(Vlty,1)
  }
  legend(legendPos,legend=legendNames,col=Vcol,pch=Vpch,title=NULL,inset = .02,lty=Vlty,ncol=1,lwd=1,cex=0.8,bg="white")
}
#gustsStatsResultsManagement(AlgoFlags,Action="load",Vcol=Vcol,Vpch=Vpch,Vlty=Vlty)

#################################################################################
# Bins and density plot
#################################################################################
data2bins=function(X=1:100,Y=NULL,xmin=min(X,na.rm=TRUE),xmax=max(X,na.rm=TRUE),N=5,step=-1,CumSum=FALSE,Integral=FALSE,BeNiceWithSingleton=F,CompressNA=F,ForceMiddle=F){
  if(step==-1){
    step=(xmax-xmin)/N
  }else{
    N=floor((xmax-xmin)/step)
  }
  s=seq(xmin,xmax,by=step)
  out = matrix(NA, N, 3)
  for(i in 1:(length(s)-1)){
    if(i==(length(s)-1)){    Id=which(X>=s[i] & X<=s[i+1])}
    else{Id=which(X>=s[i] & X<s[i+1])}
    l=length(Id)
    if((l>0 && CumSum) || (l>1)){
      out[i,1]=mean(X[Id], na.rm=TRUE)
      if(!is.null(Y)){
        out[i,2]= mean(Y[Id], na.rm=TRUE);
        out[i,3]=   sd(Y[Id], na.rm=TRUE)#/sqrt(l);
      }else{   # in case we only have one vector, we return its density of probability
        out[i,2]=l
      }
    }else{
		if(BeNiceWithSingleton && l==1){
			out[i,1]=X[Id];
			out[i,2]=Y[Id];
		}
    }
    
  }
  if(is.null(Y)){
    out[is.na(out[,2]),2]=0
    if(CumSum){
      if(!Integral){
	out[,2]=cumsum(out[,2])
      }else{
        integral=sum(out[,2],na.rm=TRUE);
        out[,2]=out[,2]/integral;
        out[,3]=out[,3]/integral
	out[,2]=cumsum(out[,2])
      }
    }
    if(!CumSum){
      if(Integral){
        integral=sum(out[,2],na.rm=TRUE)*step;
        out[,2]=out[,2]/integral;
        out[,3]=out[,3]/integral
      }
    }
  }
  if(ForceMiddle){
	out[,1]=rm.na(MoveAve(s,2))
  }
  return(out)
}

## for compatibility with my old functions
densityPlot =function(X=1:100,Densities=FALSE,Scatter=FALSE,ylab="",...){
  ylab="Density of Probability [.]"
  binsPlot(X,NULL,Densities=FALSE,Scatter=FALSE,ylab=ylab,...)
}

cumDistributionPlot =function(X=1:100,Densities=FALSE,Scatter=FALSE,CumSum=TRUE,Integral=TRUE,ylab="",...){
  ylab="Cumulative distribution [.]"
  binsPlot(X,NULL,Densities=FALSE,Scatter=FALSE,CumSum=CumSum,Integral=Integral,ylab=ylab,...)
}


binsPlot= function(X=1:100,Y=100:1,xmin=NULL,xmax=NULL,N=20,step=-1,ymin=NULL,ymax=NULL,Xgrid=20,Ygrid=20,
          color1=NULL, color2=NULL, ErrorBar=T, Densities=FALSE, BarPlot=FALSE, Scatter=T, CumSum=FALSE,Integral=TRUE,...){

  scaleXmin=FALSE;scaleXmax=FALSE;
  if(is.null(xmin)){xmin=min(X,na.rm=TRUE); scaleXmin=TRUE;}
  if(is.null(xmax)){xmax=max(X,na.rm=TRUE); scaleXmax=TRUE;}
  M=data2bins(X,Y,xmin,xmax,N,step,CumSum,Integral)
  x   = M[,1];
  y   = M[,2];
  std = M[,3];
  if(!is.null(Y)){ #case of a bin plot
    if(scaleXmin){xmin=min(x[!is.na(y)],na.rm=TRUE)}
    if(scaleXmax){xmax=max(x[!is.na(y)],na.rm=TRUE)}
  }else{ #case of a density plot
    if(scaleXmin){xmin=min(x[which(y>0)],na.rm=TRUE)}
    if(scaleXmax){xmax=max(x[which(y>0)],na.rm=TRUE)}
  }

  if(is.null(ymin)){ymin=min(y,na.rm=TRUE)}
  if(is.null(ymax)){ymax=max(y,na.rm=TRUE)}

  if(is.null(color1)){color1="blue"}
  if(is.null(color2)){color2="slategray1"}

  # Scatter : points of original data, before binning
  if(Scatter){
    plot(X,Y,axes=FALSE,xlim=c(xmin,xmax),ylim=c(ymin,ymax),xlab="",ylab="",cex=0.8,col=color2,pch=21)
    par(new=TRUE)
  }

  if(!ErrorBar){std=NULL}
  plotWithGrid(x,y,xmin,xmax,ymin,ymax,Xgrid,Ygrid,std=std,ErrorBar,BarPlot,col=color1,lwd=2,...)

  # Plotting the density of X and Y
  if(Densities){
    # density of x bin
    M=data2bins(X,NULL,xmin,xmax,N,step)
    x1=M[,1]; y1=M[,2];
    y1max=max(y1,na.rm=TRUE)
    par(new=TRUE); plot(x1,y1,xlim=c(xmin,xmax),ylim=c(0,8*y1max),axes=FALSE,col="red",pch=".",type="o",xlab="",ylab="",xaxs = "i", yaxs = "i")

    #density of y bin
    M=data2bins(Y,NULL,ymin,ymax,50,-1)
    x1=M[,1]; y1=M[,2];
    y1max=max(y1,na.rm=TRUE)
    par(new=TRUE); plot(y1,x1,ylim=c(ymin,ymax),xlim=c(0,8*y1max),axes=FALSE,col="green",pch=".",type = "o",xlab="",ylab="",xaxs = "i", yaxs = "i")
  }
}
#gustsStatsResultsManagement(DbFolder=DbFolder,Flag=AlgoFlags[9],Action="plot",Vcol=Vcol,Vpch=Vpch,Vlty=Vlty)
