########################################################################
# Dealing with time
########################################################################
#ddates=as.character(floor(M[,1]/10000)%%1000000);
#timestamp=chron(dates.=ddates,format=c("ymd"))
#~ th=floor((M[,1]%%10000)/100);
#~ tm=M[,1]%%100;
#~ ttimes=sprintf("%02d:%02d:00",th,tm)
#~ timestamp=chron(dates.=ddates,times.=ttimes,format=c("ymd","h:m:s"))
#  getTimeTicksPosition(timestamp);
#  tt=times((1:60) /60/24)
#  tt=times((1:60) /3600/24)
#  timePlotWithGrid(tt,1:60)
########################################################################

#~ 
#~ # a simple mixture of
#~ # - N( 0, 1) with weight .4
#~ # - N( 3, 1) with weight .6
#~ x <- rnorm( 1000 , 
#~ 	mean = sample( c(0, 3), size = 1000, prob = c(.4, .6),  replace = TRUE ) ) 
#~ 
#~ # axis labels always horizontal
#~ # margin smaller than default
#~ par( las = 1, mar = c(3,3,4,1)+.1 )  
#~ 
#~ # calculate histogram (but do not display)
#~ h <- hist( x, plot = FALSE, breaks = 15 )
#~ 
#~ # calculate density estimation
#~ d <- density( x ) 
#~ 
#~ # just set the scene and axes
#~ plot( h , border = NA, freq = FALSE, xlab = "", ylab = "", 
#~ 	main= "Use of clipping and translucency" ) 
#~ 
#~ # grab the limits of the region
#~ usr <- par( "usr" )
#~ ncolors <- 100
#~ dy <- ( usr[4] - usr[3] ) / ncolors
#~ 
#~ # create the colors
#~ colors <- colorRampPalette( c("yellow","orange","red") )(ncolors)
#~ 
#~ # horizontal line
#~ abline( h = axTicks(2) , col = "gray", lwd = .5 )
#~ 
#~ # for each color, clip into a region and redraw the histogram with that color
#~ for( i in 1:ncolors){
#~   clip( usr[1], usr[2], usr[3] + (i-1) * dy, usr[3] + i*dy )
#~   plot( h, add = TRUE, axes = FALSE, ylab = "", xlab = "", 
#~   	col = colors[i], border = NA, freq = FALSE)
#~ }
#~ # reset the clipping area. See ?clip
#~ do.call( clip, as.list( usr) )
#~ 
#~ # just to get the boxes right
#~ plot( h, add = TRUE, lwd = .5 , freq = FALSE, xlab = "", 
#~ 	ylab = "", axes = FALSE )
#~ 
#~ # adds a tick and translucent line to display the density estimate
#~ lines( d, lwd = 4, col = "#22222288" )
#~ 
#~ # add a rug (also using translucency)
#~ rug( x, col = "#00000088" ) 
#~ 
#~ # box around the plot
#~ box()




plotWSSample=function(tt=NULL,WS, fs=NULL,...){
	if(!inherits(tt,"times")){
		#the following assumes t in seconds
		if(is.null(tt)){
			if(is.null(fs)){stop("!!!! If you don't provide t, provide fs !!!")}
			tt=(1/fs)*(0:(length(WS)-1))
		}
		if(diff(range(tt)) <= 600){
			xlab="Time [m:s]";
			tt=tt/24/60
		}else{
			xlab="Time [h:m]";
			if(diff(range(tt)) >= 83000){
				warning("!!!  Sample too big! provide the timestamp yourself !!!")
				xlab="Time [days]";
			}
			tt=tt/24/3600
		}
		tt=times(tt,format="h:m:s")
	}
    timePlotWithGrid(tt,WS,col=4,pch="",type="o",
					xlab=xlab,
					ylab="Wind Speed [m/s]",...)

}
#plotWSSample(1:600,1:600)
#plotWSSample(WS=1:600,fs=1/600)





timePlotWithGrid=function(t,y,xaxs="r",yaxs="r",...){
	#this function assumes that t inherits of time t=times((1:60)/3600/24)
    plot.empty(t,y,xaxs=xaxs,yaxs=xaxs)
    abline(v=getTimeTicksPosition(t),h=axTicks(2),col = "lightgray", lty=3)
    par(new=T)
    plot(t,y,xaxs=xaxs,yaxs=xaxs,...)

}

	
	
plotWSdistr=function(WS,xmaxx=30,ymaxx=0.12,step=1, bonusTitl="", plotFlag=c("man","fit","manfit"),cumFlag=c("also","only","not")){
	plotFlag=plotFlag[1]
	cumFlag=cumFlag[1]

	WS_absciss=seq(0,xmaxx,0.05)

	sigma=sd(WS,na.rm=T)
	Vmean=mean(WS,na.rm=T)
	print(Vmean)

	legends="WS bins from data";
	fills="slategray1";
	############################################################################################################
	##############  Calculation : fitting with Weibull
	############################################################################################################
	# manual distribution fitting
	if(plotFlag=="man" || plotFlag=="manfit"){
		fp<-function(x) abs(gamma(1+2/x)/gamma(1+1/x)^2-1-sigma^2/Vmean^2)
		print(fp(1))
		kk=optimize(fp,c(0.1,4))$minimum
		AA=Vmean/gamma(1+1/kk)
		print(c(kk,AA))
		
		fp_Weibull<-function(x) kk/AA*(x/AA)^(kk-1)*exp( -(x/AA)^kk )
		fp_WeibullCum<-function(x) 1-exp(-(x/AA)^kk )

		legends=c(legends,sprintf("Weibull Distribution k=%.1f A=%.1fm/s",kk,AA));
		fills=c(fills,"blue")
	}
	
	# using distrfit
	if(plotFlag=="fit" || plotFlag=="manfit"){
		w = fitdistr(WS, "weibull",list(shape = 1.1, scale = Vmean/0.9),lower = c(0.9, 1),upper=c(4,12),method="L-BFGS-B")
		k=w[[1]][1];
		A=w[[1]][2];
		legends=c(legends,sprintf("Weibull Distribution k=%.1f A=%.1fm/s",k,A));
		fills=c(fills,"black")
	}
	############################################################################################################
	##############  plotting the WS distribution
	############################################################################################################
	if(cumFlag!="only"){
		title=paste("Wind speed distribution",bonusTitl,sep="")
		beginPlot(title)
		densityPlot(WS,xmin=0,xmax=xmaxx,step=step,ymin=0,ymax=ymaxx,Xgrid=10,Ygrid=10,
		  ErrorBar=FALSE, BarPlot=TRUE,
		  color1="slategray1",
		  main=title,
		  xlab="Wind speed [m/s]",
		  yxaxs = "i", yaxs = "i")

		if(plotFlag=="man" || plotFlag=="manfit"){
			par(new=TRUE)
			plot(WS_absciss,fp_Weibull(WS_absciss),axes=FALSE,
				xlim=c(0,xmaxx),ylim=c(0,ymaxx),
				xaxs="i",yaxs="i",
				xlab="",ylab="",pch=".",type = "o",cex=2,lwd=2, 
				col="blue")
		}
		if(plotFlag=="fit" || plotFlag=="manfit"){
			par(new=TRUE)
			lines(WS_absciss, dweibull(WS_absciss, k,A), lwd=2, col="black",
				xlim=c(0,xmaxx),ylim=c(0,ymaxx),
				xaxs="i",yaxs="i")
		}
	   	legend("topright", legends ,fill=fills,inset = .05,ncol=1,lwd=1,cex=0.8,bg="white",lty=0.5,pch="")
		box()
		endPlot()
	}
	
	
	############################################################################################################
	##############  plotting the WS Cumulative distribution
	############################################################################################################
	if(cumFlag=="only" || cumFlag=="also"){
		legends=c("WS bins from data",sprintf("Weibull Function k=%.1f A=%.1fm/s",kk,AA))
		fills=c("slategray1","black")
		title=paste("Wind speed cumulative distribution ",bonusTitl,sep="")
		beginPlot(title)
		cumDistributionPlot(WS,xmin=0,xmax=xmaxx,step=1,ymin=0,ymax=1,Xgrid=10,Ygrid=10,
		BarPlot=TRUE,
		color1="slategray1",
		main=title,
		xlab="Wind speed [m/s]")
			legend("right", legends,fill=fills,title=NULL,inset = .06,ncol=1,lwd=1,cex=0.8,bg="white",lty=0.5,pch="")
		par(new=TRUE)
		plot(WS_absciss,fp_WeibullCum(WS_absciss),axes=FALSE,xlim=c(0,xmaxx),ylim=c(0,1),xlab="",ylab="",pch=".",type = "o",cex=2,lwd=2, col="black",yaxs="i",xaxs="i",frame.plot=TRUE)
	endPlot()
	}
}
#plotWSdistr(WS)













mywindrose=function (x, y = NULL, breaks = NULL, bins = 12, increment = 10, 
    main = "Wind Rose", cir.ind = 0.05, fill.col = NULL, plot.mids = TRUE, 
    mids.size = 1.2, osize = 0.1, axes = TRUE, ticks = TRUE, 
    tcl = 0.025, tcl.text = -0.15, cex = 1, digits = 2, num.ticks = 12, 
    xlim = c(-1.2, 1.2), ylim = c(-1.2, 1.2), uin = NULL, tol = 0.04, 
    right = FALSE, shrink = NULL, label.freq = FALSE, calm = c("0", 
        "NA"), doffset=T,...) 
{
    calm <- match.arg(calm)
    circles <- function(rad, sector = c(0, 2 * pi), lty = 2, 
        col = "white", border = NA, fill = FALSE) {
        values <- seq(sector[1], sector[2], by = (sector[2] - 
            sector[1])/360)
        x <- rad * cos(values)
        y <- rad * sin(values)
        if (fill) {
            polygon(x, y, xpd = FALSE, lty = lty, col = col, 
                border = border)
        }
        lines(x, y, col = 1, lty = lty)
    }
    if (!is.null(cir.ind) && (cir.ind > 1 | cir.ind <= 0)) {
        cir.ind <- 0.05
        warning("'cir.ind' must be in (0, 1]")
    }
    if (any(is.null(y))) {
        if (is.data.frame(x) && NCOL(x) == 2) {
            y <- x[, 2]
            x <- x[, 1]
        }
        else {
            stop("'y' can not be 'NULL' if 'x' is not a dataframe with 2 columns (direction and magnitude) ")
        }
    }
    else {
        y <- as.vector(y)
        if (length(y) != length(x)) 
            stop("'x' and 'y' must have the same length")
        if (is.data.frame(x)) {
            if (NCOL(x) > 1) 
                x <- x[, 1]
        }
    }
    no <- length(x)
    if (calm == "NA") {
        calmcalm <- sum(is.na(x))
        notcalm <- !is.na(x)
    }
    else {
        calmcalm <- sum(x == calm)
        notcalm <- x != calm
    }
    x <- x[notcalm]
    y <- y[notcalm]
    ok <- complete.cases(x, y)
    x <- x[ok]
    y <- y[ok]
    if (length(y) == 0) {
        warning("No observations (at least after removing missing values and calm winds)")
        return(NULL)
    }
    result <- list()
    result$x <- x
    result$y <- y
    xcircularp <- attr(as.circular(x), "circularp")
    type <- xcircularp$type
    units <- xcircularp$units
    template <- xcircularp$template
    modulo <- xcircularp$modulo
    zero <- xcircularp$zero
    rotation <- xcircularp$rotation
    op <- par(mar = c(1, 1, 2, 1))
    mai <- par("mai")
    on.exit(par(op))
    midx <- 0.5 * (xlim[2] + xlim[1])
    xlim <- midx + (1 + tol) * 0.5 * c(-1, 1) * (xlim[2] - xlim[1])
    midy <- 0.5 * (ylim[2] + ylim[1])
    ylim <- midy + (1 + tol) * 0.5 * c(-1, 1) * (ylim[2] - ylim[1])
    oldpin <- par("pin") - c(mai[2] + mai[4], mai[1] + mai[3])
    xuin <- oxuin <- oldpin[1]/diff(xlim)
    yuin <- oyuin <- oldpin[2]/diff(ylim)
    if (is.null(uin)) {
        if (yuin > xuin) 
            xuin <- yuin
        else yuin <- xuin
    }
    else {
        if (length(uin) == 1) 
            uin <- uin * c(1, 1)
        if (any(c(xuin, yuin) < uin)) 
            stop("uin is too large to fit plot in")
        xuin <- uin[1]
        yuin <- uin[2]
    }
    xlim <- midx + oxuin/xuin * c(-1, 1) * diff(xlim) * 0.5
    ylim <- midy + oyuin/yuin * c(-1, 1) * diff(ylim) * 0.5


    if (any(is.null(breaks))) {
        step <- 2 * pi/bins
		########################################################################################################################################
		if(doffset){
			doff=-step/2;
        }else{
			doff=0;
        }
        breaks <- circular(seq(0+doff, 2 * pi+doff, by = step), units = "radians")
		print(breaks)
		########################################################################################################################################

    }
    else {
        breaks <- as.circular(breaks)
    }

   
    breaks <- conversion.circular(breaks, units = "radians", 
        zero = 0, rotation = "counter", modulo = "2pi")
    attr(breaks, "class") <- attr(breaks, "circularp") <- NULL
    breaks <- sort(unique(breaks))
    if (breaks[1] != 0) {
        breaks <- c(breaks[length(breaks)] - 2 * pi, breaks)
    }
    else {
        breaks <- c(breaks, 2 * pi)
    }

	print(breaks)

    attr(breaks, "class") <- attr(breaks, "circularp") <- NULL
    bins <- length(breaks) - 1
    step <- diff(breaks)

    x <- conversion.circular(x, units = "radians", zero = 0,  rotation = "counter", modulo = "2pi")
    attr(x, "class") <- attr(x, "circularp") <- NULL
    
    x[x >= breaks[bins + 1]] <- x[x >= breaks[bins + 1]] - 2 * pi
    plot(c(-1.2, 1.2), c(-1.2, 1.2), xlab = "", ylab = "", main = main,  xaxt = "n", yaxt = "n", pch = " ", xlim = xlim, ylim = ylim)
    counts <- hist(x, breaks = breaks, plot = FALSE, right = right)$counts

	print(counts)

    mids <- breaks[1:bins] + step/2
    if (plot.mids) {
        for (i in 1:bins) {
            lines(c(0, mids.size * cos(mids[i])), c(0, mids.size*sin(mids[i])), lty = 2)
        }
    }


	###########################################################################################################
	# maxlength.orig <- sqrt(max(counts/step)/length(x) + osize^2)
    maxlength.orig <- max(counts/step)/length(x)
    if (is.null(shrink)) {
        maxlength <- shrink <- maxlength.orig
    }
    else {
        maxlength <- shrink
    }
    J <- ceiling(max(y, na.rm = TRUE)/increment)
    if (any(is.null(fill.col))) {
        if (J%%2 == 0) {
            fill.col <- c("blue", "red")
        }
        else {
            fill.col <- c("red", "blue")
        }
    }
    fill.col <- rep(fill.col, length.out = bins)


    OUT <- matrix(NA, nrow = J, ncol = bins)
    OUT[J, ] <- counts
    for (j in J:1) {
        data1 <- x[y <= j * increment]
        OUT[j, ] <- counts <- hist(data1, breaks = breaks, plot = FALSE, 
            right = right)$counts
        for (i in 1:bins) {
            w1 <- breaks[i]
            w2 <- breaks[i + 1]
            if (counts[i]) {
			   ###########################################################################################################
               # rad <- sqrt(counts[i]/(step[i] * length(x)) + osize^2)/maxlength
                rad <- counts[i]/(step[i] * length(x))/maxlength
            }
            else {
                rad <- 0
            }
            xx <- rad * c(0, cos(w1), cos(w2), 0)
            yy <- rad * c(0, sin(w1), sin(w2), 0)
            polygon(xx, yy, xpd = FALSE, col = fill.col[j], border = NA)
            lines(xx[1:2], yy[1:2])
            lines(xx[3:4], yy[3:4])
            circles(rad = rad, sector = c(w1, w2), fill = TRUE, 
                lty = 1, col = fill.col[j], border = NA)
        }
    }
    m <- dim(OUT)[1]
    new <- OUT
    if (m > 1) {
        for (i in 2:m) {
            new[i, ] <- OUT[i, ] - OUT[i - 1, ]
        }
    }

    
    if (!is.null(cir.ind)) {
        equalstep <- max(abs(diff(step))) <= 10 * .Machine$double.eps
		###########################################################################################################
        #max.plt <- maxlength.orig^2 - osize^2
        max.plt <- maxlength.orig
        
        if (equalstep & label.freq) 
            max.plt <- max.plt * step[1]

        cir.ind <- min(cir.ind, max.plt)
        max.plt <- floor(max.plt/cir.ind) * cir.ind
        max.plt <- seq(cir.ind, max.plt, by = cir.ind)

		
        
        if (equalstep & label.freq) {
			###########################################################################################################
            #rad <- sqrt(max.plt/step[1] + osize^2)/maxlength
            rad <- max.plt/step[1]/maxlength
        }
        else {
			###########################################################################################################
            #rad <- sqrt(max.plt + osize^2)/maxlength
            rad <- max.plt/maxlength
        }

		print("rad")
        print(rad)
        if (equalstep & label.freq) {
            text(0, rad, paste(round(max.plt * 100, digits = digits), 
                "%", sep = ""), cex = 0.9 * cex, pos = 3, font = 3, 
                offset = 0.2)
        }
        else {
            text(0, rad, paste(round(max.plt, digits = digits), 
                sep = ""), cex = 0.9 * cex, pos = 3, font = 3, 
                offset = 0.2)
        }

        for (i in 1:length(rad)) {
            circles(rad[i])
        }
    }
    else {
        circles(1)
    }

#~ 
    
    circles(osize, fill = TRUE)
    if (ticks) {
        at <- (0:num.ticks)/num.ticks * 2 * pi
        if (rotation == "clock") 
            at <- -at
        at <- at + zero
        ticks.circular(circular(x = at, type = "angles", units = "radians", 
            modulo = "asis", zero = zero, rotation = rotation), 
            tcl = tcl)
    }
    if (axes) {
        axis.circular(units = units, template = template, zero = zero, 
            rotation = rotation, digits = digits, cex = cex, 
            tcl = tcl, tcl.text = tcl.text)
    }
    
    OUT <- round(new/sum(OUT[m, ]), 3)
    colnamesout <- rep("", bins)
    breaks <- conversion.circular(circular(breaks), units = units)
    mids <- conversion.circular(circular(mids), units = units)
    for (i in 1:bins) {
        if (right) {
            colnamesout[i] <- paste("(", round(breaks[i], digits = digits), 
                ", ", round(breaks[i + 1], digits = digits), 
                "]", sep = "")
        }
        else {
            colnamesout[i] <- paste("[", round(breaks[i], digits = digits), 
                ", ", round(breaks[i + 1], digits = digits), 
                ")", sep = "")
        }
    }
    colnames(OUT) <- colnamesout
    rownames(OUT) <- paste("(", 0:(J - 1) * increment, ",", 1:J * 
        increment, "]", sep = "")
    result$table <- OUT
    result$number.obs <- no
    result$number.calm <- calmcalm
    result$breaks <- breaks
    result$mids <- mids
    result$shrink <- shrink
    result$call <- match.call()
    invisible(result)
}









